/*
 * smtp.h
 * This file is part of LibSMTPc
 *
 * Copyright (C) 2022 - Félix Arreola Rodríguez
 *
 * LibSMTPc is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * LibSMTPc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LibSMTPc; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef __SMTP_H__
#define __SMTP_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#define SMTP_WANT_NONE			0x000
#define SMTP_WANT_POLL_IN		0x001
#define SMTP_WANT_POLL_OUT		0x002

enum {
	SMTP_ERROR = -1,
	SMTP_POLL = 0,
	SMTP_SUCCESS = 1
};

enum {
	SMTP_STATE_CLOSED = 0,
	SMTP_STATE_TRYING_CONNECT,
	SMTP_STATE_BANNER,
	SMTP_STATE_HELO,
	SMTP_STATE_READY,
	SMTP_STATE_AUTH_LOGIN,
	SMTP_STATE_AUTH_LOGIN_USER,
	SMTP_STATE_AUTH_LOGIN_PASS,
	SMTP_STATE_CMD,
	SMTP_STATE_DATA,
	SMTP_STATE_SENDING_DATA,
	
	SMTP_STATE_CLOSING,
};

typedef struct _SMTPSession SMTPSession;

typedef void (*SMTPLineFunc) (SMTPSession *session, const char *line, void *data);

SMTPSession *smtp_session_create (void);
int smtp_session_auth_login (SMTPSession *session, const char *user, const char *passwd);
int smtp_session_connect (SMTPSession *session, struct sockaddr *server, socklen_t addr_len, const char *helo_name);
void smtp_session_close_and_clean (SMTPSession *session);
int smtp_session_send_command (SMTPSession *session, char *command);
int smtp_session_send_close (SMTPSession *session);
int smtp_session_send_data (SMTPSession *session, FILE *stream);

int smtp_session_continue (SMTPSession *session, int what);
int smtp_session_get_state (SMTPSession *session);
int smtp_session_get_poll (SMTPSession *session);
int smtp_session_get_socket (SMTPSession *session);
int smtp_session_get_last_code (SMTPSession *session);
char * smtp_session_get_last_message (SMTPSession *session);
void smtp_session_free (SMTPSession *session);

void smtp_session_multiline_func (SMTPSession *session, SMTPLineFunc func, void *data);

#endif /* __SMTP_H__ */

